<?php

namespace App\Http\Controllers\Campaign;

use App\Budget;
use App\Campaign;
use App\Company;
use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Company\CompanyController;
use App\Http\Controllers\Controller;
use App\Provider;
use App\Reseller;
use App\User;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class CampaignController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:campaigns.index')->only('index');
        $this->middleware('can:campaigns.campaingsAdminSales')->only('campaingsAdminSales');
        $this->middleware('can:campaigns.getAllCampaignsEditables')->only('getAllCampaignsEditables');
        $this->middleware('can:campaigns.getAllMyCampaignsEditables')->only('getAllMyCampaignsEditables');
        $this->middleware('can:campaigns.getAllCampaignsEditablesAdminSales')->only('getAllCampaignsEditablesAdminSales');
        $this->middleware('can:campaigns.customerCampaigns')->only('customerCampaigns');
        $this->middleware('can:campaigns.myCampaigns')->only('myCampaigns');
        $this->middleware('can:campaigns.store')->only('store');
        $this->middleware('can:campaigns.show')->only('show');
        $this->middleware('can:campaigns.update')->only('update');
        $this->middleware('can:campaigns.destroy')->only('destroy');
        $this->middleware('can:campaigns.duplicate')->only('duplicate');
        $this->middleware('can:campaigns.disableCampaign')->only('disableCampaign');
        $this->middleware('can:campaigns.activateCampaign')->only('activateCampaign');
        $this->middleware('can:campaigns.addDateImpactsCampaign')->only('addDateImpactsCampaign');
        $this->middleware('can:campaigns.search')->only('search');
        $this->middleware('can:campaigns.searchMe')->only('searchMe');
        $this->middleware('can:campaigns.getPDFBudget')->only('getPDFBudget');
    }
    /**
     * Listar todas la campañas de la base de datos.
     *      -Superadministradores
     *      -Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll( Campaign::select('id', 'name')->get() );
        }
        $campaigns = Campaign::orderBy( 'id', 'desc' )
                             ->with('seller')
                             ->with('customer')
                             ->with('country')
                             ->withCount('walls')
                             ->withCount('witnesses')
                             ->paginate(8);

        foreach ($campaigns as $key => $campaign) {
            $impacts = $campaign->walls()
                            ->select('i.*')
                            ->join('impacts_data as iD','iD.wall_id','=','walls.id')
                            ->join('impacts as i','i.id','=','iD.impact_id')
                            ->orderBy('i.id', 'desc')
                            ->get()
                            ->unique('id');

            if (count($impacts) > 0) {
                $campaign['impacts_count'] = count($impacts);
            } else {
                $campaign['impacts_count'] = 0;
            }
        }

        return $this->showAll(Collection::make($campaigns));
    }

    /**
     * Metodo para buscar una campaña de todos
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
     *
     * @return \Illuminate\Http\Response
     */
    public function search()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $campaigns;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $campaigns = Campaign::orderBy('id', 'DESC')
                         ->with('seller')
                         ->with('customer')
                         ->with('country')
                         ->withCount('walls')
                         ->withCount('witnesses')
                         ->orWhere('id', 'LIKE', '%'.request('word').'%')
                         ->orWhere('name', 'LIKE', '%'.request('word').'%')
                         ->paginate(8);
        }

        if (auth()->user()->hasRole('salesmanager')) {

            $campaigns = Campaign::select('campaigns.*')
                            ->join('users as u','u.id','=','campaigns.user_registry_id')
                            ->join('users as uv','uv.id','=','campaigns.customer_campaign_id')
                            ->where('u.admin_registry_id', auth()->user()->id)
                            ->where( function($q){
                                $q->orWhere('campaigns.id', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('campaigns.name', 'LIKE', '%'.request('word').'%');
                            })
                            ->with('seller')
                            ->with('customer')
                            ->with('country')
                            ->withCount('walls')
                            ->paginate(8);
        }

        return $this->showAll(Collection::make($campaigns), 201);
    }

    /**
     * Metodo para buscar una campaña mia
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Sellercosts Seller
        ** Customercosts customer
     *
     * @return \Illuminate\Http\Response
     */
    public function searchMe()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $campaigns;

        $campaigns = Campaign::where('user_registry_id', auth()->user()->id)
                        ->where( function($q){
                            $q->orWhere('id', 'LIKE', '%'.request('word').'%')
                              ->orWhere('name', 'LIKE', '%'.request('word').'%');
                        })
                        ->with('seller')
                        ->with('customer')
                        ->with('country')
                        ->withCount('walls')
                        ->withCount('witnesses')
                        ->paginate(8);

        return $this->showAll(Collection::make($campaigns), 201);
    }

    /**
     * Listar todas la campañas de la base de datos que son editables.
     *      -Superadministradores
     *      -Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function getAllCampaignsEditables()
    {
        $campaings = Campaign::select('id', 'name')
                             ->orderBy( 'id', 'desc' )
                             ->where( 'editable', Campaign::EDITABLE )
                             ->get();
        return $this->showAll(Collection::make($campaings));
    }

    /**
     * Listar todas las campañas editables de un usuario logeado
     *      -Superadministradores
            -Administradores
            -Salesmanager
            -Sellers costs
            -Sellers
            -Customer costs
            -Customer
     *
     * @return \Illuminate\Http\Response
     */
    public function getAllMyCampaignsEditables()
    {
        $campaigns = Campaign::select('id', 'name')
                             ->where(
                              [
                                'user_registry_id' => auth()->user()->id,
                                'editable' => Campaign::EDITABLE
                              ])
                              ->orderBy( 'id', 'desc' )
                              ->get();

        return $this->showAll(Collection::make($campaigns));
    }

    /**
     * Listar todas las campañas relacionadas con un administrador ventas.
     *      - Administrador Ventas
     *
     * @return \Illuminate\Http\Response
     */
    public function campaingsAdminSales()
    {
        if (request()->get('per_page') === 'all') {
            $campaings = Campaign::select('campaigns.*')
                        ->join('users as u','u.id','=','campaigns.user_registry_id')
                        ->join('users as uv','uv.id','=','campaigns.customer_campaign_id')
                        ->where('u.admin_registry_id', auth()->user()->id)
                        ->get();
            return $this->showAll($campaigns);
        }
        // Listar todas las camapñas de usuarios relacionados con un administrador ventas
        $campaings = Campaign::select('campaigns.*')
                        ->join('users as u','u.id','=','campaigns.user_registry_id')
                        ->join('users as uv','uv.id','=','campaigns.customer_campaign_id')
                        ->where('u.admin_registry_id', '=', auth()->user()->id)
                        ->orderBy( 'id', 'desc' )
                        ->with('seller')
                        ->with('customer')
                        ->with('country')
                        ->withCount('walls')
                        ->withCount('witnesses')
                        ->paginate(8);
        return $this->showAll(Collection::make($campaings));
    }

    /**
     * Listar todas las campañas editables administrador ventas
     *      -Salesmanager
     *
     * @return \Illuminate\Http\Response
     */
    public function getAllCampaignsEditablesAdminSales()
    {
        $campaigns = Campaign::select('campaigns.id', 'campaigns.name')
                        ->join('users as u','u.id','=','campaigns.user_registry_id')
                        ->join('users as uv','uv.id','=','campaigns.customer_campaign_id')
                        ->where(
                            [
                                'u.admin_registry_id' => auth()->user()->id,
                                'campaigns.editable' => Campaign::EDITABLE
                            ]
                        )->get();

        return $this->showAll(Collection::make($campaigns));
    }

    /**
     * Listar todas las campañas de un usuario logeado.
     *      -Superadministradores
            -Administradores
            -Salesmanager
            -Sellers costs
            -Sellers
            -Customer costs
            -Customer
     * @return \Illuminate\Http\Response
     */
    public function myCampaigns()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll( auth()->user()->campaigns_sellers );
        }

        $campaigns = auth()->user()
                           ->campaigns_sellers()
                           ->orderBy( 'id', 'desc' )
                           ->with('customer')
                           ->with('seller')
                           ->with('country')
                           ->withCount('walls')
                           ->withCount('witnesses')
                           ->paginate(8);

        return $this->showAll(Collection::make($campaigns));
    }

    /**
     *      Listar todas las campañas de un cliente logeado.
     *          - Customer cost
                - Customer
     *      @return \Illuminate\Http\Response
     */
    public function customerCampaigns()
    {
        $campaigns = auth()->user()->campaigns_customers()
                                   ->orderBy( 'id', 'desc' )
                                   ->with('seller')
                                   ->with('customer')
                                   ->with('country')
                                   ->withCount('walls')
                                   ->withCount('witnesses')
                                   ->paginate( 8 );
        return $this->showAll( Collection::make($campaigns) );
    }

    /**
     * Store a newly created resource in storage.
            -Superadministradores
            -Administradores
            -Salesmanager
            -Sellers costs
            -Sellers
            -Customer costs
            -Customer
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        /*
            Realizar las validaciones para los usuarios
            para saber cuales es vendedor y cual es cliente

            Validacion para poder asignar una campaña a otro usuario

            Validaciones estado de la campaña
        */
        $rules = [
            'user_registry_id'  =>  ['required', 'integer'],
            'country_id'        =>  ['required', 'integer'],
            'name'              =>  ['required', 'string'],
            'comentary'         =>  ['string'],
            'investment'        =>  ['numeric'],
            'status'            =>  ['required', 'in:'.Campaign::ACTIVA.','.Campaign::PAUSADA.','.Campaign::PROPUESTA.','.Campaign::AUTORIZADA.','.Campaign::FINALIZADA.','.Campaign::CANCELADA],
            'customer_campaign_id' => ['required', 'integer'],
            'country'              => ['string']
        ];
        $this->validate(request(), $rules);

        $fields = request()->all();

        if (!User::find(request('user_registry_id')) || !User::find(request('customer_campaign_id'))) {
            return $this->errorResponse('Usuario inexistente', 404);
        }

        if (request('status') == Campaign::ACTIVA || request('status') == Campaign::FINALIZADA) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) && !auth()->user()->hasRole( 'salesmanager' ) ) {

                $fields['status'] = Campaign::PROPUESTA;

            }
        }

        $campaign = Campaign::create($fields);
        return $this->showOne($campaign, 201);
    }

    /**
     *      Mostrar una campaña en especifico.
     *
     *          -Superadministradores
                -Administradores
                -Salesmanager
                -Sellers costs
                -Sellers
                -Customer costs
                -Customer
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Campaign $campaign)
    {
        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        if (
            auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator') ||
            auth()->user()->hasRole('salesmanager') || auth()->user()->hasRole('sellercosts') ||
            auth()->user()->hasRole('customercosts')
        ) {
            // Pantallas
            $walls_campaign = $campaign->walls->load('country')->load('circuit')->makeHidden('local_currency');

            // ids de las empresas de todas las pantallas
            $companiesIds = [];
            foreach ($walls_campaign as $key => $wall) {
                if (!in_array($wall->company_id, $companiesIds)) {
                    array_push($companiesIds, $wall->company_id);
                }
            }

            // Obenetmos los multiplicadores de los proveedores con los ids
            $companiesMultiplier = Provider::whereIn( 'company_id', $companiesIds )->get();

            // Obtenemos multiplicador seller
            $multiplier_seller = Reseller::where( 'company_id', $campaign->seller->company_id)->first();
            $multiplier_seller  = $multiplier_seller ? $multiplier_seller->multiplier : 1;

            // Porcentajes del proveedor
            $percentage_provider =  \DB::table('company_country')
                ->select('company_id', 'country_id', 'discount', 'increase')
                ->whereIn('company_id', $companiesIds)->get();

            //Porcentage de la empresa del vendedor
            $percentage_seller = \DB::table('company_country')
                ->select('company_id', 'country_id', 'discount', 'increase')
                ->where('company_id', $campaign->seller->company_id)
                ->get();

            foreach ($walls_campaign as $key => $wall) {

                $wall->discount = 0;
                $wall->increase = 0;

                if (count($companiesMultiplier) !== 0) {
                    foreach ($companiesMultiplier as $key => $company) {
                        if ($company->company_id == $wall->company_id) {

                            $cost = $wall->local_currency * $company->multiplier;
                            $price_seller = $cost * $multiplier_seller;
                            $price_provider_discount = 0;
                            $price_provider_increase = 0;

                            if (count($percentage_provider) > 0) {
                                foreach ($percentage_provider as $key => $pP) {
                                    if ($pP->company_id == $wall->company_id && $pP->country_id == $wall->country_id) {
                                        $price_provider_discount = $price_seller - ($price_seller * ($pP->discount / 100));
                                        $price_provider_increase = $price_provider_discount * (($pP->increase / 100) + 1);
                                        $wall->price = $price_provider_increase;
                                        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                                            $wall->discount = $pP->discount;
                                            $wall->increase = $pP->increase;
                                        }
                                    }
                                }
                            }

                            if (count($percentage_seller) > 0) {
                                foreach ($percentage_seller as $key => $pS) {
                                    if ($pS->country_id == $wall->country_id) {
                                        $price_seller_discount = $price_provider_increase - ($price_provider_increase * ($pS->discount / 100));
                                        $price_seller_increase = $price_seller_discount * (($pS->increase / 100) + 1);
                                        $wall->price = $price_seller_increase;
                                        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                                            $wall->discount_reseller_company = $pS->discount;
                                            $wall->increase_reseller_company = $pS->increase;
                                        }
                                    }
                                }
                            }

                        }
                    }
                } else {
                    $cost = $wall->local_currency;
                    $price_seller = $cost * $multiplier_seller;
                    $price_provider_discount = 0;
                    $price_provider_increase = 0;

                    if (count($percentage_provider) > 0) {
                        foreach ($percentage_provider as $key => $pP) {
                            if ($pP->company_id == $wall->company_id && $pP->country_id == $wall->country_id) {
                                $price_provider_discount = $price_seller - ($price_seller * ($pP->discount / 100));
                                $price_provider_increase = $price_provider_discount * (($pP->increase / 100) + 1);
                                $wall->price = $price_provider_increase;
                                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                                    $wall->discount = $pP->discount;
                                    $wall->increase = $pP->increase;
                                }
                            }
                        }
                    }

                    if (count($percentage_seller) > 0) {
                        foreach ($percentage_seller as $key => $pS) {
                            if ($pS->country_id == $wall->country_id) {
                                $price_seller_discount = $price_provider_increase - ($price_provider_increase * ($pS->discount / 100));
                                $price_seller_increase = $price_seller_discount * (($pS->increase / 100) + 1);
                                $wall->price = $price_seller_increase;
                                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                                    $wall->discount_reseller_company = $pS->discount;
                                    $wall->increase_reseller_company = $pS->increase;
                                }
                            }
                        }
                    }
                }
            }

            return $this->showOne($walls_campaign);
        }

        $campaign = $campaign->walls
            ->load('country')
            ->load('circuit')
            ->makeHidden('local_currency');

        return $this->showOne($campaign);
    }

    /**
     * Update the specified resource in storage.

            -Superadministradores
            -Administradores
            -Salesmanager
            -Sellers costs
            -Sellers
            -Customer costs
            -Customer
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Campaign $campaign)
    {

        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        $rules = [
            'user_registry_id'  =>  ['integer'],
            'country_id'        =>  ['integer'],
            'name'              =>  ['string'],
            'comentary'         =>  ['string'],
            'type_of_campaign'  =>  ['in:'.Campaign::ADSERVER.','.Campaign::DSP],
            'investment'        =>  ['numeric'],
            'start'             =>  ['regex:/^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/'],
            'finish'            =>  ['regex:/^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/'],
            'status'            =>  ['in:'.Campaign::ACTIVA.','.Campaign::PAUSADA.','.Campaign::PROPUESTA.','.Campaign::AUTORIZADA.','.Campaign::FINALIZADA.','.Campaign::CANCELADA],
            'customer_campaign_id' => ['integer'],
            'country'              => ['string'],
            'total_days_of_the_campaign' => ['integer']
        ];

        $this->validate(request(), $rules);

        if (request('user_registry_id')) {
            if (!User::find(request('user_registry_id'))) {
                return $this->errorResponse('Nonexistent user', 404);
            } else {
                $campaign->user_registry_id = request('user_registry_id');
            }
        }

        if (request('name')) {
            $campaign->name = request('name');
        }

        if (request('comentary')) {
            $campaign->comentary = request('comentary');
        }

        if (request('total_days_of_the_campaign')) {
            $campaign->total_days_of_the_campaign = request('total_days_of_the_campaign');
        }

        if (request('type_of_campaign')) {
            $campaign->type_of_campaign = request('type_of_campaign');
        }

        if (request('investment')) {
            $campaign->investment = request('investment');
        }

        if (request('start')) {
            $campaign->start = request('start');
        }

        if (request('finish')) {
            $campaign->finish = request('finish');
        }

        if (request('status')) {
            if (request('status') == Campaign::ACTIVA || request('status') == Campaign::FINALIZADA) {
                if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) && !auth()->user()->hasRole( 'salesmanager' ) ) {

                    $campaign->status = Campaign::PROPUESTA;

                } else {
                    $campaign->status = request('status');
                }
            } else {
                $campaign->status = request('status');
            }
        }

        if (request('customer_campaign_id')) {
            if (!User::find(request('customer_campaign_id'))) {
                return $this->errorResponse('Nonexistent user', 404);
            } else {
                $campaign->customer_campaign_id = request('customer_campaign_id');
            }
        }

        if (request('country')) {
            $campaign->country = request('country');
        }

        $campaign->save();

        return $this->showOne($campaign);
    }

    /**
    Realizar las validaciones correspondientes al eliminar una campaña
    dicha campaña lleva relacion con usuarios, pantallas, poligonos, etc
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Campaign $campaign)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }


        $campaign->polygons()->delete();
        $campaign->pointsOfInterest()->delete();
        $campaign->walls()->detach();
        $campaign->budget()->delete();
        $campaign->delete();

        return $this->showOne($campaign);
    }

    /*
            -Superadministradores
            -Administradores
            -Salesmanager
            -Sellers costs
            -Sellers
            -Customer costs
            -Customer
    */
    public function duplicate(Campaign $campaign) {

        $duplicated_campaign = $campaign->replicate();

        $duplicated_campaign['name'] = 'copia '.date("Y-m-d").' '.$duplicated_campaign['name'];

        $duplicated_campaign['editable'] = Campaign::EDITABLE;

        $duplicated_campaign->push();

        $walls_campaign = $campaign->walls;
        $points_of_interest_campaign = $campaign->pointsOfInterest;
        $polygons_campaign = $campaign->polygons;

        if (sizeof($walls_campaign) > 0) {
            foreach ($walls_campaign as $wall) {
                $duplicated_campaign->walls()->syncWithoutDetaching($wall->id);
            }
        }

        if (sizeof($points_of_interest_campaign) > 0) {
            foreach ($points_of_interest_campaign as $point_of_interest_campaign) {
                $duplicated_campaign->pointsOfInterest()->create($point_of_interest_campaign->toArray());
            }
        }

        if (sizeof($polygons_campaign) > 0) {
            foreach ($polygons_campaign as $polygon_campaign) {
                $duplicated_campaign->polygons()->create($polygon_campaign->toArray());
            }
        }

        $campaign_duplicate = Campaign::with('seller')
                                      ->with('customer')
                                      ->with('country')
                                      ->where( 'id', $duplicated_campaign->id )
                                      ->first();

        return $this->showOne($campaign_duplicate);
    }

    /**
     *  Metodo para desactivar una campaña
     *  SUPERADMINISTRADORES y ADMINISTRADORES
     *
     *
     *
     *
     */
    public function disableCampaign( Campaign $campaign ) {
        $campaign->editable = Campaign::NOT_EDITABLE;
        $campaign->save();
        return $this->showOne($campaign);
    }

    /**
     *  Metodo para activar una campaña
     *  SUPERADMINISTRADORES y ADMINISTRADORES
     *
     *
     *
     *
     */
    public function activateCampaign( Campaign $campaign ) {
        $campaign->editable = Campaign::EDITABLE;
        $campaign->save();
        return $this->showOne($campaign);
    }

    /**
     *  Metodo para agregar una fecha de audiencia a una campaña
     *  SUPERADMINISTRADORES y ADMINISTRADORES
     *
     *
     *
     *
     */
    public function addDateImpactsCampaign( Campaign $campaign ) {

        $rules = [
            'date_range'  =>  ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $campaign->date_impacts = request('date_range');
        $campaign->save();
        return $this->showOne($campaign);
    }

    /**
     *  Metodo para descargar el presupuesto de una campaña
        SUPERADMINISTRADORES y ADMINISTRADORES
        Vendedores con costos, administradores de ventas,
        cliente con costos
     *
     *
     *
     *
     */
    public function getPDFBudget( Campaign $campaign )
    {
        $walls_campaign = $campaign->walls->load('country');
        $budget = $campaign->budget;

        $countryCampaign = $campaign->country()->first();

        $company_seller = Company::where('id', $campaign->seller->company_id )
            ->firstOrFail();

        $companiesIds = [];
        foreach ($walls_campaign as $key => $wall) {
            if (!in_array($wall->company_id, $companiesIds)) {
                array_push($companiesIds, $wall->company_id);
            }
        }

        $companiesMultiplier = Provider::whereIn( 'company_id', $companiesIds )->get();

        $multiplierReseller = Reseller::where('company_id', $campaign->seller->company_id)->first();
        $multiplierReseller = $multiplierReseller ? $multiplierReseller->multiplier : 1;

        // Porcentajes del proveedor
        $percentage_provider =  \DB::table('company_country')
            ->select('company_id', 'country_id', 'discount', 'increase')
            ->whereIn('company_id', $companiesIds)->get();

        //Porcentage de la empresa del vendedor
        $percentage_seller = \DB::table('company_country')
            ->select('company_id', 'country_id', 'discount', 'increase')
            ->where('company_id', $campaign->seller->company_id)
            ->get();

        $budgetDataJson = json_decode($budget->data_json, true);
        $budgetDiscount = $budget->discount;

        $wall_data = [];

        foreach ($budgetDataJson as $key => $budget) {
            $wall_data[$budget['wall_id']] = $budget['time'];
        }

        $priceTotal = 0;
        $priceTotalUSD = 0;

        $tbody = '';
        $title = 'General';

        foreach ($walls_campaign as $key => $wall) {

            $countryWall = Country::find($wall->country_id);

            $priceWallMultiplier = $wall->local_currency * $multiplierReseller;

            foreach ($companiesMultiplier as $key => $providerMultiplier) {
                if ($wall->company_id == $providerMultiplier->company_id) {
                    $priceWallMultiplier = $priceWallMultiplier * $providerMultiplier->multiplier;
                }
            }

            // Precio Multiplicadores y Costos extras
//            $priceWall = ($wall_data[$wall->id] * $priceWallMultiplier) + $wall->production_cost + $wall->installation_cost;
//            $priceWallUSD = $priceWall / $countryWall->dollar_change;

            $priceWallPercentages = 0;

            if (count($percentage_provider) > 0) {
                foreach ($percentage_provider as $key => $pP) {
                    if ($pP->company_id == $wall->company_id && $pP->country_id == $wall->country_id) {
                        $discount = $pP->discount / 100;
                        $increase = ($pP->increase / 100) + 1;
                        $priceWallPercentages = $priceWallMultiplier - ($priceWallMultiplier * $discount);
                        $priceWallPercentages = $priceWallPercentages * $increase;
                    }
                }
            }

            if (count($percentage_seller) > 0) {
                foreach ($percentage_seller as $key => $pS) {
                    if($pS->country_id == $wall->country_id) {
                        $discount = $pS->discount / 100;
                        $increase = ($pS->increase / 100) + 1;

                        $priceWallPercentages = $priceWallPercentages - ($priceWallPercentages * $discount);
                        $priceWallPercentages = $priceWallPercentages * $increase;
                    }
                }
            }

            $priceWallTotal = ($priceWallPercentages * $wall_data[$wall->id]) + $wall->production_cost + $wall->installation_cost;
            $priceWallTotalUSD = $priceWallTotal / $countryWall->dollar_change;

            #$priceTotal += $priceWallTotalUSD * $countryCampaign->dollar_change;
            $priceTotalUSD += $priceWallTotal / $countryWall->dollar_change;


            $tbody .= '
                        <tr>
                            <td>DOOH</td>
                            <td>'.$wall->country->country_name.'</td>
                            <td>'.$wall->provider_wall_id.'</td>
                            <td>
                                <ul>
                                    <li>Precio: '.number_format($priceWallPercentages, 2).'</li>
                                    <li>Costo Producción: '.number_format($wall->production_cost, 2).'</li>
                                    <li>Costo Instalación: '.number_format($wall->installation_cost, 2).'</li>
                                </ul>
                            </td>
                            <td>'.$wall_data[$wall->id].' Meses</td>
                            <td>'.number_format($priceWallTotal, 2).'</td>
                            <td>'.number_format($priceWallTotalUSD, 2).'</td>
                        </tr>
                    ';
        }

        //  Price local currency campaign
        $priceTotalLCCampaign = $priceTotalUSD * $countryCampaign->dollar_change;
        $priceWithTax = 0;
        $priceUSDWithTax = 0;

        // Calculate discount price
        $priceWithDiscount = 0;
        $priceUSDWithDiscount = 0;
        $priceWithDiscountTax = 0;
        $priceUSDWithDiscountTax = 0;


        if ($budgetDiscount && $budgetDiscount > 0) {
            $priceWithDiscount = $priceTotalLCCampaign - ( $priceTotalLCCampaign * ( $budgetDiscount / 100 ) );
            $priceUSDWithDiscount = $priceTotalUSD - ( $priceTotalUSD * ( $budgetDiscount / 100 ) );

            $priceWithDiscountTax = $priceWithDiscount * ( ($countryCampaign->tax / 100) + 1 );
            $priceUSDWithDiscountTax = $priceUSDWithDiscount * ( ($countryCampaign->tax / 100) + 1 );
        } else {
            $priceWithTax = $priceTotalLCCampaign * ( ($countryCampaign->tax / 100) + 1 );
            $priceUSDWithTax = $priceTotalUSD * ( ($countryCampaign->tax / 100) + 1 );
        }

        $tableDiscount = '';

        if ($priceWithDiscount && $priceWithDiscount > 0) {
            $tableDiscount = '
            <td class="border-cero"></td>
            <td class="border-cero"></td>
            <td class="border-cero"></td>
            <td class="border-cero">Descuento</td>
            <td class="border-cero">%'.$budgetDiscount.'</td>
            <td class="border-cero">$'.number_format($priceWithDiscount, 2).'</td>
            <td class="border-cero">$'.number_format($priceUSDWithDiscount, 2).'</td>';
        }

        $tableTax = '';

        if ($priceWithDiscount && $priceWithDiscount > 0) {
            $tableTax = '
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none">Total</td>
                        <td style="border: none">$'.number_format($priceWithDiscountTax, 2).'</td>
                        <td style="border: none">$'.number_format($priceUSDWithDiscountTax, 2).'</td>';
        } else {
            $tableTax = '
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none"></td>
                        <td style="border: none">Total</td>
                        <td style="border: none">$'.number_format($priceWithTax, 2).'</td>
                        <td style="border: none">$'.number_format($priceUSDWithTax, 2).'</td>';
        }

        $html = '
                <!DOCTYPE HTML>
                <html lang="es">
                    <head>
                        <style type="text/css">
                        .nav {
                            width:100%;
                            background:#212020;
                        }
                        .logo {
                            padding:8px;
                            margin-right: 10px;
                            padding-top: 20px;
                        }

                        #customers {
                          font-family: Arial, Helvetica, sans-serif;
                          border-collapse: collapse;
                          width: 100%;
                        }

                        #customers td, #customers th {
                          border: 1px solid #ddd;
                          padding: 8px;
                        }

                        #customers tr:nth-child(even){background-color: #f2f2f2;}

                        #customers tr:hover {background-color: #ddd;}

                        #customers th {
                          padding-top: 12px;
                          padding-bottom: 12px;
                          text-align: left;
                          background-color: #4CAF50;
                          color: white;
                        }

                        .info-campana {
                            margin-top: 5px;
                        }

                        .nombre-campana {
                            color:white;
                            margin-left:40%;
                            font-size:20px;
                            font-weight: bold;
                        }

                        .table-orden-compra {
                            width:100%;
                            font-size:10px;
                        }

                        .tbody-orden-compra {
                            font-size: 12px;
                        }

                        .border-cero {
                            border:0px solid white;
                            background: white;
                        }

                        .border-cero-derecha {
                            border-top:0px;
                            border-left:0px;
                            border-bottom:0px;
                        }
                        </style>
                    </head>
                    <body>
                        <div class="nav">
                            <div class="logo">
                                <img src="https://panel.oohgroup.com/assets/images/default/oohgroup.png">
                                <span class="nombre-campana">
                                    '.ucwords($campaign->name).'
                                </span>
                            </div>
                        </div>
                        <div class="info-campana">
                            <p> Detalles campaña </p>
                            <ul>
                                <li>
                                    Campaña: '.ucwords($campaign->name).'
                                </li>
                                <li>
                                    Campaña ID: '.$campaign->id.'
                                </li>
                                <li>
                                    País: '.$countryCampaign->country_name.'
                                </li>
                            </ul>
                        </div>
                        <div class="table-orden-compra">
                            <table id="customers">
                                <thead>
                                    <tr>
                                        <th>PLATAFORMA</th>
                                        <th>PAÍS</th>
                                        <th>MURO ID</th>
                                        <th>COSTOS</th>
                                        <th>TIEMPO</th>
                                        <th>TOTAL <small>(Moneda Local Pantalla)</small> </th>
                                        <th>PRECIO <small>(USD)</small></th>
                                    </tr>
                                </thead>

                                <tbody class="tbody-orden-compra">
                                '.$tbody.'
                                </tbody

                                <tbody>
                                    <tr style="font-size: 13px">
                                        <th>Totales <small>(Moneda local campaña)</small> </th>
                                        <th></th>
                                        <th></th>
                                        <th></th>
                                        <th></th>
                                        <th>$'.number_format($priceTotalLCCampaign, 2).'</th>
                                        <th>$'.number_format($priceTotalUSD, 2).'</th>
                                    </tr>
                                </tbody>

                                <tbody>
                                    <tr style="font-size: 13px">
                                        '.$tableDiscount.'
                                    </tr>
                                </tbody>


                            </table>

                            <table id="customers" style="font-size: 13px">
                                <tbody>
                                    <tr>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none"></td>
                                        <td style="border: none">IVA</td>
                                        <td style="border: none">%'.$countryCampaign->tax.'</td>
                                    </tr>
                                </tbody>
                                <tbody>
                                    <tr>
                                        '.$tableTax.'
                                    </tr>
                                </tbody>
                            </table>

                        </div>
                    </body>
                </html>
            ';

        $pdf = \PDF::loadHTML($html)->setPaper('a4', 'landscape')->setWarnings(false);

        $name = $campaign->id.'.pdf';

        $pdf->save( public_path().'/pdfs/temporary/'.$name );

        return $this->showMessage($name, 200);
    }

}
