<?php

namespace App\Http\Controllers\Impact;

use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Impact;
use App\ImpactData;
use App\Wall;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class ImpactController extends ApiController
{
    /**
     * Listar todos los Impactos
        Roles
        Superadministradores
        Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $impacts = Impact::orderBy('id', 'desc')->paginate(8);
        return $this->showAll(Collection::make($impacts));
    }

    /**
     * Listar todos los Impactos filtrado por paises y estados
        Roles
        ** Todos los roles, menos el suspendido
     *
     * @return \Illuminate\Http\Response
     */
    public function filteredImpacts()
    {
        $rules = [
                    'countries_ids.*' => ['numeric'],
                    'states.*' => ['string']
                ];

        $this->validate(request(), $rules);

        $impacts;

        if (request('countries_ids') && !request('states')) {
            $impacts = Impact::select('date_range', \DB::raw('count(*) as total'))
                                            ->groupBy('date_range')
                                            ->whereIn('country_id', request('countries_ids'))
                                            ->get();
        }

        if (request('countries_ids') && request('states')) {
            $impacts = Impact::select('date_range', \DB::raw('count(*) as total'))
                                            ->groupBy('date_range')
                                            ->whereIn('country_id', request('countries_ids'))
                                            ->whereIn('state', request('states'))
                                            ->get();
        }

        if (!request('countries_ids') && !request('states')) {
            $impacts = Impact::select('date_range', \DB::raw('count(*) as total'))->groupBy('date_range')->get();
        }

        return $this->showAll(Collection::make($impacts));
    }

    /**
     * Listar todos los Impactos de un impacto
        Roles
        Superadministradores
        Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function getImpacts(Impact $impact)
    {
        if (request()->get('per_page') === 'all') {
            $impacts = $impact->impacts;
            return $this->showAll( Collection::make($impacts) );
        }
        $impacts = $impact->impacts()->paginate(8);
        return $this->showAll( Collection::make($impacts) );
    }

    /**
     * Listar todos los Impactos de un rango de fechas
        Roles
        Todos
     *
     * @return \Illuminate\Http\Response
     */
    public function getImpactsDataRange()
    {
        $rules = [
                    'date_range' => ['string', 'required'],
                    'countries_ids.*' => ['numeric'],
                    'states.*' => ['string']
                ];

        $this->validate(request(), $rules);

        $impactsIDs;

        if (request('countries_ids') && !request('states')) {

            $impactsIDs = Impact::select('id')
                                ->whereIn('country_id', request('countries_ids'))
                                ->where('date_range', request('date_range'))
                                ->get()
                                ->pluck('id');
        }

        if (request('countries_ids') && request('states')) {
            $impactsIDs = Impact::select('id')
                                ->whereIn('country_id', request('countries_ids'))
                                ->whereIn('state', request('states'))
                                ->where('date_range', request('date_range'))
                                ->get()
                                ->pluck('id');
        }

        if (!request('countries_ids') && !request('states')) {
            $impactsIDs = Impact::select('id')
                                ->where('date_range', request('date_range'))
                                ->get()
                                ->pluck('id');
        }

        $data_impacts = ImpactData::whereIn('impact_id', $impactsIDs)->get();

        return $this->showAll( Collection::make($data_impacts) );
    }

    /**
     * Obtener ultimos 6 impactos de una pantalla
        Roles
        Superadministradores
        Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function getImpactsWall(Wall $wall)
    {
        $impacts = ImpactData::orderBy('created_at', 'DESC')
                            ->with('impact')
                            ->where( 'wall_id', $wall->id )
                            ->take(5)
                            ->get();
        // return $screen->impacts->load('impact')->latest()->take(5)->get();

        // $date = date("Y-m-d");
        // $lastMonthsDate = date("Y-m-d", strtotime($date."- 6 month"));
        // $lastMonthsDate = date("Y-m-d", strtotime($lastMonthsDate."- ".(date('j') - 1)."  day"));

        // $currentMonth = date("Y-m-d", strtotime($date."- ".(date('j') - 1)."  day"));

        // $impacts = Impact::orderBy( 'date', 'asc' )
        //                  ->where('wall_id', $screen->id)
        //                  ->whereDate('date', '>=', $lastMonthsDate)
        //                  ->whereDate('date', '<', $currentMonth)
        //                  ->get();

        return $this->showAll($impacts);
    }

    /**
     * Obtener Lista de impactos de una pantalla
        Roles
        Todos los roles
     *
     * @return \Illuminate\Http\Response
     */
    public function getImpactsListWall(Wall $wall)
    {
        $impacts_list = Impact::select('impacts.id', 'impacts.date_range')
                            ->join('impacts_data', 'impacts.id', '=', 'impacts_data.impact_id')
                            ->where('impacts_data.wall_id', $wall->id)
                            ->get();
        return $this->showAll($impacts_list);
    }

    /**
     * Obtener impactos de un rango de fechas de una pantalla
        Roles
        Todos los roles
     *
     * @return \Illuminate\Http\Response
     */
    public function getImpactsWallDataRange(Impact $impact, Wall $wall)
    {
        $impacts = ImpactData::orderBy('created_at', 'DESC')
                             ->where(
                                    [
                                        'impact_id' => $impact->id,
                                        'wall_id' => $wall->id
                                    ]
                                )
                                ->first();
                                // ->get();
        return $this->showOne($impacts);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
       $rules = [
            'country_id'         =>   ['required', 'numeric'],
            'state'              =>   ['required', 'string'],
            'name'               =>   ['required', 'string'],
            'date_range'         =>   ['required', 'string'],
            'file'               =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $impact = Impact::create(request()->all());

        $reader = \PhpOffice\PhpSpreadsheet\IOFactory::createReader("Xlsx");
        $spreadsheet = $reader->load(public_path().'/files/temporary/'.request('file'));
        $spreadsheet->setActiveSheetIndex(0);
        $numRows = $spreadsheet->setActiveSheetIndex(0)->getHighestRow();

        for($i = 2; $i <= $numRows; $i++){
            $wall_id         =   $spreadsheet->getActiveSheet()->getCell('A'.$i)->getCalculatedValue();
            $male_sex        =   $spreadsheet->getActiveSheet()->getCell('B'.$i)->getCalculatedValue();
            $female_sex      =   $spreadsheet->getActiveSheet()->getCell('C'.$i)->getCalculatedValue();
            $ages_13_17      =   $spreadsheet->getActiveSheet()->getCell('D'.$i)->getCalculatedValue();
            $ages_18_25      =   $spreadsheet->getActiveSheet()->getCell('E'.$i)->getCalculatedValue();
            $ages_26_40      =   $spreadsheet->getActiveSheet()->getCell('F'.$i)->getCalculatedValue();
            $ages_41_55      =   $spreadsheet->getActiveSheet()->getCell('G'.$i)->getCalculatedValue();
            $ages_55         =   $spreadsheet->getActiveSheet()->getCell('H'.$i)->getCalculatedValue();
            $sl_a            =   $spreadsheet->getActiveSheet()->getCell('I'.$i)->getCalculatedValue();
            $sl_b            =   $spreadsheet->getActiveSheet()->getCell('J'.$i)->getCalculatedValue();
            $sl_c            =   $spreadsheet->getActiveSheet()->getCell('K'.$i)->getCalculatedValue();
            $sl_d            =   $spreadsheet->getActiveSheet()->getCell('L'.$i)->getCalculatedValue();
            $sl_e            =   $spreadsheet->getActiveSheet()->getCell('M'.$i)->getCalculatedValue();
            $total           =   $spreadsheet->getActiveSheet()->getCell('N'.$i)->getCalculatedValue();
            $frequency       =   $spreadsheet->getActiveSheet()->getCell('O'.$i)->getCalculatedValue();
            $reach           =   $spreadsheet->getActiveSheet()->getCell('P'.$i)->getCalculatedValue();

            ImpactData::create([
                'impact_id'       =>    $impact->id,
                'wall_id'         =>    $wall_id,
                'male_sex'        =>    $male_sex,
                'female_sex'      =>    $female_sex,
                'ages_13_17'      =>    $ages_13_17,
                'ages_18_25'      =>    $ages_18_25,
                'ages_26_40'      =>    $ages_26_40,
                'ages_41_55'      =>    $ages_41_55,
                'ages_55'         =>    $ages_55,
                'sl_a'            =>    $sl_a,
                'sl_b'            =>    $sl_b,
                'sl_c'            =>    $sl_c,
                'sl_d'            =>    $sl_d,
                'sl_e'            =>    $sl_e,
                'total'           =>    $total,
                'frequency'       =>    $frequency,
                'reach'           =>    $reach
            ]);
        }

        Storage::disk('temporaryFiles')->delete( request( 'file' ) );

        return $this->showMessage('ok', 200);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function storeImpact()
    {
        $rules = [
            'impact_id'     => ['required', 'numeric'],
            'wall_id'       => ['required', 'numeric'],
            'male_sex'      => ['required', 'numeric'],
            'female_sex'    => ['required', 'numeric'],
            'ages_13_17'    => ['required', 'numeric'],
            'ages_18_25'    => ['required', 'numeric'],
            'ages_26_40'    => ['required', 'numeric'],
            'ages_41_55'    => ['required', 'numeric'],
            'ages_55'       => ['required', 'numeric'],
            'sl_a'          => ['required', 'numeric'],
            'sl_b'          => ['required', 'numeric'],
            'sl_c'          => ['required', 'numeric'],
            'sl_d'          => ['required', 'numeric'],
            'sl_e'          => ['required', 'numeric'],
            'total'         => ['required', 'numeric'],
            'reach'         => ['required', 'numeric']
        ];

        $this->validate(request(), $rules);

        if (!Wall::find( request('wall_id') )) {
            return $this->errorResponse('Nonexistent wall', 400);
        }

        $impact = ImpactData::create(request()->all());

        return $this->showOne($impact, 200);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Impact $impact)
    {
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Impact  $impact
     * @return \Illuminate\Http\Response
     */
    public function update(ImpactData $impact)
    {
        $rules = [
            'impact_id'         => ['numeric'],
            'wall_id'         => ['numeric'],
            'male_sex'          => ['numeric'],
            'female_sex'        => ['numeric'],
            'ages_13_17'        => ['numeric'],
            'ages_18_25'        => ['numeric'],
            'ages_26_40'        => ['numeric'],
            'ages_41_55'        => ['numeric'],
            'ages_55'           => ['numeric'],
            'sl_a'              => ['numeric'],
            'sl_b'              => ['numeric'],
            'sl_c'              => ['numeric'],
            'sl_d'              => ['numeric'],
            'sl_e'              => ['numeric'],
            'total'             => ['numeric'],
            'reach'             => ['numeric']
        ];

        $this->validate(request(), $rules);

        if (!Wall::find( request('wall_id') )) {
            return $this->errorResponse('Nonexistent wall', 400);
        }

        $impact->fill(request()->only([
            'impact_id',
            'wall_id',
            'male_sex',
            'female_sex',
            'ages_13_17',
            'ages_18_25',
            'ages_26_40',
            'ages_41_55',
            'ages_55',
            'sl_a',
            'sl_b',
            'sl_c',
            'sl_d',
            'sl_e',
            'total',
            'reach'
        ]));

        $impact->save();

        return $this->showOne($impact);
    }

    /**
     * Remove the specified resource from storage.
        Roles
        ** Superadministradores
        ** Administradores
     *
     * @param  \App\Impact  $impact
     * @return \Illuminate\Http\Response
     */
    public function destroy(Impact $impact)
    {
        $impact->impacts()->delete();
        $impact->delete();
        return $this->showOne($impact);
    }

    /**
     * Remove the specified resource from storage.
        Roles
        ** Superadministradores
        ** Administradores
     *
     * @param  \App\Impact  $impact
     * @return \Illuminate\Http\Response
     */
    public function destroyImpact(ImpactData $impact)
    {
        $impact->delete();
        return $this->showOne($impact);
    }

    /**
     * Subir archivo de impactos
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadFile() {
        $file = request()->file( 'file' );
        $input = array( 'file' => $file );
        $rules = [ 'file' => ['required', 'mimes:xls,xlsx'] ];
        $validate = Validator::make($input, $rules);
        if ($validate->fails())
        {
            return $this->errorResponse('Only xls or xlsx', 422);
        }
        $data['file'] = request('file')->store('', 'temporaryFiles');
        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Eliminar archivo de impactos
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteFile() {
        $rules = [
            'file'        =>    ['required', 'string']
        ];
        $this->validate( request(), $rules );
        Storage::disk('temporaryFiles')->delete( request( 'file' ) );
        return $this->showMessage('Deleted File', 200);
    }
}
